/***************************************************************************
* File Name: hal430_TmpSnsr.c
* Processor(s): MSP430F16x (tested), MSP430F14x (not tested)
* Compiler: Code Composer Studio V4 (4.2.5.00005)
* 
* Copyright  2013 Playing With Fusion, Inc.
* SOFTWARE LICENSE AGREEMENT: This code is released under the MIT License. 
* 
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
* **************************************************************************
* REVISION HISTORY:
* Author			Date		Comments
* J. Steinlage		2013Nov30	Original version
* 
* **************************************************************************
* ADDITIONAL NOTES:
* This file contains functions to configure, read and organize data from
* the MAX31865 board. 
***************************************************************************/

#include "msp430x16x.h"
#include "msp430_typeDef.h"
#include "pwf_an0002_defines.h"
#include "hal430_TmpSnsr.h"
#include "hal430_spi.h"
#include "pwf_an0002_struct.h"

// thermocouple channels
extern struct var_MAX31865_ch temp_RTD0;

void MAX31865_init(void)
{
	HalSpiInit_an0002(); 		// initialize MAX31865-specific SPI HAL
	MAX31865_config();
}

void update_RTDs(void)	// configured for use with MAX31855K-4CH board
{
	struct var_MAX31865_ch *struct_ptr;	// pointer to a structure

	struct_ptr = &temp_RTD0;				// set struct_ptr for RTD 0 structure
	MAX31865_update_temp(struct_ptr);		// update RTD 0
	
	if(0x02 == (P1IN & 0x02)){struct_ptr->RTD_type = 1;}	// pull RTD type pin hi to select PT100
	else{struct_ptr->RTD_type = 2;}							// pull RTD type pin low to select PT1000
	
	// config '0' indicates issue with communications. if detected, re-set config 
	// the MAX31865 requires configuration prior to valid results will be read
	if(0 == struct_ptr->config){MAX31865_config();}
	
	// clear faults that might be present. Keep in mind some faults (voltage above/below thresholds)
	// may immediately re-set if condition is present
	if(0 != struct_ptr->status){MAX31865_clear_fault();} // clear fault(s)

	
}

void MAX31865_config(void)
{
	
	signed int temp;
	// set CS low
	P2OUT &= ~RTD0_CS;
	
	// Write byte to IC indicating 'write' and 'config address'
	U0TXBUF = 0x80;
	
	complete();
	temp = U0RXBUF;

	// Write byte to IC configuring 'config' register, clearing fault status
	// Vbias on, Auto convert, cont, 2/4 wire, no fault cycle, clear faults, 60Hz -> 0xC2
	U0TXBUF = 0xC2;					
	
	complete();
	temp = U0RXBUF;
	P2OUT |= RTD0_CS;
	
	volatile unsigned int i;
	for(i=0; i<10000; i++)
	{}
}

void MAX31865_clear_fault(void)
{
	
	signed int temp;
	// set CS low
	P2OUT &= ~RTD0_CS;
	
	// Write byte to IC indicating 'write' and 'config address'
	U0TXBUF = 0x80;
	
	complete();
	temp = U0RXBUF;

	// set fault status clear bit in config register
	U0TXBUF |= 0x02;					
	
	complete();
	temp = U0RXBUF;
	P2OUT |= RTD0_CS;
	
	volatile unsigned int i;
	for(i=0; i<10000; i++)
	{}
}


void MAX31865_update_temp(struct var_MAX31865_ch *RTD_ptr)
{
	// note: the MAX31865 provides an output with 15 bits of resolution and a nominal 0.03125 deg C resolution
	uint16 RTD_Conf, RTD_MSBs, RTD_LSBs, HFT_MSB, HFT_LSB, LFT_MSB, LFT_LSB, Fault_Status;
	static sint16 temp;
	static uint32 temp2;

	//Start by asserting ONE chip select pin 'low' (easy with a single board, must remember this if more than one)
	//RTD_ID can be used if reading more than one RTD chip with this code.
	P2OUT &= ~RTD0_CS;

	//Tell the chip what to start with... 
	U0TXBUF = 0x00;					// Config register
	complete();						// Wait until all 8 bits have been received
	temp = U0RXBUF;					// Store this data in 'trash'
	
	// Get Config data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	RTD_Conf = U0RXBUF;				// Store this data in the lo-byte variable.
	
	// Get RTD_MSBs data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	RTD_MSBs = U0RXBUF;				// Store this data in the lo-byte variable.

	// Get RTD_LSBs data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	RTD_LSBs = U0RXBUF;				// Store this data in the lo-byte variable.

	// Get HFT_MSB data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	HFT_MSB = U0RXBUF;				// Store this data in the lo-byte variable.
	
	// Get HFT_LSB data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	HFT_LSB = U0RXBUF;				// Store this data in the lo-byte variable.

	// Get LFT_MSB data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	LFT_MSB = U0RXBUF;				// Store this data in the lo-byte variable.

	// Get LFT_LSB data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	LFT_LSB = U0RXBUF;				// Store this data in the lo-byte variable.

	// Get Fault_Status data byte from the chip
	U0TXBUF = 0x00;					// Send clocks to the chip, no command.
	complete();						// Wait until all 8 bits have been received. 
	Fault_Status = U0RXBUF;			// Store this data in the lo-byte variable.
	
	P2OUT |= RTD0_CS;				// Set CS back high

	/* update RTD struct with relevant data */
	RTD_ptr->status = Fault_Status;	// update status
	
	RTD_ptr->config = RTD_Conf;		// update config (feedback)
	
	temp = HFT_MSB<<8;				// update High Temp Fault Threshold
	temp = temp | HFT_LSB;
	RTD_ptr->HFT_val = temp;
	
	temp = LFT_MSB<<8;				// update Low Temp Fault Threshold
	temp = temp | LFT_LSB;
	RTD_ptr->LFT_val = temp;
	
	temp = RTD_MSBs<<8;				// update RTD raw value
	temp |= RTD_LSBs;
	RTD_ptr->raw_value = temp>>1;	// Shift 1 right as reg bit 0 is not part of temp
	
	// update RTD resistance 
		// (Rrtd = (ADC Code x Rref)/2^15)... raw value, single shift left * Rref gives uint32, 15 bits fraction
	if(1 == RTD_ptr->RTD_type){temp2 = (uint32)RTD_ptr->raw_value*400;}
	else{temp2 = (uint32)RTD_ptr->raw_value*4000;}
	RTD_ptr->rtd_res = temp2;
	
    /*
     * The European standard, also known as the DIN or IEC standard, is considered the world-wide
	standard for platinum RTDs. This standard, IEC751, requires the RTD to have a rated resistance 
	at 0degC (100ohm for PT100, 1000ohm for PT1000) and a temperature coefficient of resistance
	(TCR) of 0.00385 ohm/ohm/degC between 0 and 100degC.
	
	0.00385 ohm/ohm (RTD) /deg C * 100 ohm (RTD) -> 0.385 ohm/deg C
	---> (res - 100) * 0.385 = temp (PT100)
	
	0.00385 ohm/ohm (RTD) /deg C * 1000 ohm (RTD) -> 3.85 ohm/deg C
	---> (res - 1000) * 3.85 = temp (PT1000)
	*/
	// update RTD temperature
    //sint16 rtd_temp;	/* temperature of RTD... sint16,  5 bits used for fraction  (+/- 1024 deg C)*/
    static uint32 tmp_res1;
	static sint32 tmp_res2;
	static sint32 tmp_sign;

	if(1 == RTD_ptr->RTD_type){						// 1 = PT100, 2 = PT1000
	tmp_res1 = RTD_ptr->rtd_res>>1;					// get to 14 bits of fraction to avoid overflow at max resistance
	tmp_res2 = (tmp_res1 - 1638400);				// subtract 100 ohm offset @0c (100*2^14 = 1638400)
	tmp_res2 = tmp_res2 * 385;						// multiply by IEC751 std RTD slope (0.00385 ohm/ohm/degC)... mult by 1000 for precision
	tmp_res2 = tmp_res2 / 1000;						// divide final answer by 1000 to maintain 2^-14 scaling
	
	tmp_sign = tmp_res2 & 0x80000000;				// remember sign
	tmp_res2 &= 0x7FFFFFFF;							// get everything but sign into reg
	tmp_res2 = tmp_res2 >> 9;						// right shift by 9. should put fraction bits at '5' to reduce to 16 bit number
	RTD_ptr->rtd_temp = tmp_res2 | (tmp_sign>>16);	// apply sign bit
	}
	else{											// not PT100, must be 2 = PT1000
	tmp_res1 = RTD_ptr->rtd_res>>5;					// get to 10 bits of fraction to avoid overflow at max resistance
	tmp_res2 = (tmp_res1 - 1024000);				// subtract 1000 ohm offset @0c (1000*2^10 = 1024000)
	tmp_res2 = tmp_res2 * 385;						// multiply by IEC751 std RTD slope (0.00385 ohm/ohm/degC)... mult by 1000 for precision
	tmp_res2 = tmp_res2 / 1000;						// divide final answer by 1000 to maintain 2^-10 scaling
	
	tmp_sign = tmp_res2 & 0x80000000;				// remember sign
	tmp_res2 &= 0x7FFFFFFF;							// get everything but sign into reg
	tmp_res2 = tmp_res2 >> 5;						// right shift by 5. should put fraction bits at '5' to reduce to 16 bit number
	RTD_ptr->rtd_temp = tmp_res2 | (tmp_sign>>16);	// apply sign bit
	}

}
